@@ -13,6 +13,7 @@ Parse and translate XML Schema regular expressions to Python regex syntax.
 import re
 from sys import maxunicode
 
+from ..helpers import OCCURRENCE_INDICATORS
 from .unicode_subsets import RegexError, UnicodeSubset, unicode_subset
 from .character_classes import I_SHORTCUT_REPLACE, C_SHORTCUT_REPLACE, CharacterClass
 
@@ -21,10 +22,10 @@ INVALID_HYPHEN_PATTERN = re.compile(r'[^\\]-[^\\]-[^\\]')
 DIGITS_PATTERN = re.compile(r'\d+')
 QUANTIFIER_PATTERN = re.compile(r'{\d+(,(\d+)?)?}')
 FORBIDDEN_ESCAPES_NOREF_PATTERN = re.compile(
-    r'(?<!\\)\\(U[0-9a-fA-F]{8}|u[0-9a-fA-F]{4}|x[0-9a-fA-F]{2}|o{\d+}|\d+|A|Z|z|B|b|o)'
+    r'(?<!\\)\\(U[\da-fA-F]{8}|u[\da-fA-F]{4}|x[\da-fA-F]{2}|o{\d+}|\d+|A|Z|z|B|b|o|0\d{2})'
 )
 FORBIDDEN_ESCAPES_REF_PATTERN = re.compile(
-    r'(?<!\\)\\(U[0-9a-fA-F]{8}|u[0-9a-fA-F]{4}|x[0-9a-fA-F]{2}|o{\d+}|A|Z|z|B|b|o)'
+    r'(?<!\\)\\(U[\da-fA-F]{8}|u[\da-fA-F]{4}|x[\da-fA-F]{2}|o{\d+}|A|Z|z|B|b|o|0\d{2})'
 )
 
 
@@ -44,7 +45,10 @@ def translate_pattern(pattern: str, flags: int = 0, xsd_version: str = '1.0',
     :param anchors: if `True` supports ^ and $ anchors, otherwise the translated \
     pattern is anchored to its boundaries and anchors are treated as normal characters.
     """
-    def parse_character_class():
+    pos: int
+    msg: str
+
+    def parse_character_class() -> CharacterClass:
         nonlocal pos
         nonlocal msg
 
@@ -149,16 +153,22 @@ def translate_pattern(pattern: str, flags: int = 0, xsd_version: str = '1.0',
                 msg = "invalid quantifier {!r} at position {}: {!r}"
                 raise RegexError(msg.format(ch, pos, pattern))
 
+            if regex and regex[-1] in ('^', r'(?<!\n\Z)^', '$', r'$(?!\n\Z)'):
+                # ^{n} or ${n} allowed but useless. Invalid im Python re
+                # so incapsulate '^'/'$' inside a non-capturing group.
+                regex[-1] = f'(?:{regex[-1]})'
+
             regex.append(match.group())
             pos += len(match.group())
-            if not lazy_quantifiers and pos < pattern_len and pattern[pos] in ('?', '+', '*'):
-                msg = "unexpected meta character {!r} at position {}: {!r}"
-                raise RegexError(msg.format(pattern[pos], pos, pattern))
+            if pos < pattern_len and pattern[pos] in '?+*':
+                if not lazy_quantifiers or pattern[pos] != '?':
+                    msg = "unexpected meta character {!r} at position {}: {!r}"
+                    raise RegexError(msg.format(pattern[pos], pos, pattern))
             continue  # pragma: no cover
 
         elif ch == '(':
-            if pattern[pos:pos + 2] == '(?':
-                msg = "invalid '(?...)' extension notation ad position {}: {!r}"
+            if pattern[pos:pos + 2] == '(?' and pattern[pos:pos + 3] != '(?:':
+                msg = "invalid '(?...)' extension notation at position {}: {!r}"
                 raise RegexError(msg.format(pos, pattern))
 
             total_groups += 1
@@ -177,15 +187,19 @@ def translate_pattern(pattern: str, flags: int = 0, xsd_version: str = '1.0',
             nested_groups -= 1
             regex.append(ch)
 
-        elif ch in ('?', '+', '*'):
+        elif ch in OCCURRENCE_INDICATORS:
             if pos == 0:
                 msg = "unexpected quantifier {!r} at position {}: {!r}"
                 raise RegexError(msg.format(ch, pos, pattern))
-            elif lazy_quantifiers:
-                pass
-            elif pos < pattern_len - 1 and pattern[pos + 1] in ('?', '+', '*', '{'):
-                msg = "unexpected meta character {!r} at position {}: {!r}"
-                raise RegexError(msg.format(pattern[pos + 1], pos + 1, pattern))
+            elif pos < pattern_len - 1 and pattern[pos + 1] in '?+*{':
+                if not lazy_quantifiers or pattern[pos + 1] != '?':
+                    msg = "unexpected meta character {!r} at position {}: {!r}"
+                    raise RegexError(msg.format(pattern[pos + 1], pos + 1, pattern))
+
+            if regex and regex[-1] in ('^', r'(?<!\n\Z)^', '$', r'$(?!\n\Z)'):
+                # ^*/^+/^? or $*/$+/$? allowed but useless. Invalid im Python re
+                # so incapsulate '^'/'$' inside a non-capturing group.
+                regex[-1] = f'(?:{regex[-1]})'
 
             regex.append(ch)
 
